﻿/*  
    DynamicMethodDispatcher and MuticastDynamicMethodDispatcher classes:
    Generic class DynamicMethodDispatcher dispatches a call to a generic delegate isntance found by a dictionary key
    Generic class MuticastDynamicMethodDispatcher is a multi-case version of DynamicMethodDispatcher.
   
    Test/Demo Application: System.Windows.Forms, Windows
    
    Copyright (C) 2006-2011 by Sergey A Kryukov
    http://www.SAKryukov.org
*/
namespace TestMessageDispatching {

    internal enum WindowsMessage : int {
        NULL = 0x00,
        CREATE = 0x01,
        DESTROY = 0x02,
        MOVE = 0x03,
        SIZE = 0x05,
        ACTIVATE = 0x06,
        SETFOCUS = 0x07,
        KILLFOCUS = 0x08,
        ENABLE = 0x0A,
        SETREDRAW = 0x0B,
        SETTEXT = 0x0C,
        GETTEXT = 0x0D,
        GETTEXTLENGTH = 0x0E,
        PAINT = 0x0F,
        CLOSE = 0x10,
        QUERYENDSESSION = 0x11,
        QUIT = 0x12,
        QUERYOPEN = 0x13,
        ERASEBKGND = 0x14,
        SYSCOLORCHANGE = 0x15,
        ENDSESSION = 0x16,
        SYSTEMERROR = 0x17,
        SHOWWINDOW = 0x18,
        CTLCOLOR = 0x19,
        WININICHANGE = 0x1A,
        SETTINGCHANGE = 0x1A,
        DEVMODECHANGE = 0x1B,
        ACTIVATEAPP = 0x1C,
        FONTCHANGE = 0x1D,
        TIMECHANGE = 0x1E,
        CANCELMODE = 0x1F,
        SETCURSOR = 0x20,
        MOUSEACTIVATE = 0x21,
        CHILDACTIVATE = 0x22,
        QUEUESYNC = 0x23,
        GETMINMAXINFO = 0x24,
        PAINTICON = 0x26,
        ICONERASEBKGND = 0x27,
        NEXTDLGCTL = 0x28,
        SPOOLERSTATUS = 0x2A,
        DRAWITEM = 0x2B,
        MEASUREITEM = 0x2C,
        DELETEITEM = 0x2D,
        VKEYTOITEM = 0x2E,
        CHARTOITEM = 0x2F,

        SETFONT = 0x30,
        GETFONT = 0x31,
        SETHOTKEY = 0x32,
        GETHOTKEY = 0x33,
        QUERYDRAGICON = 0x37,
        COMPAREITEM = 0x39,
        COMPACTING = 0x41,
        WINDOWPOSCHANGING = 0x46,
        WINDOWPOSCHANGED = 0x47,
        POWER = 0x48,
        COPYDATA = 0x4A,
        CANCELJOURNAL = 0x4B,
        NOTIFY = 0x4E,
        INPUTLANGCHANGEREQUEST = 0x50,
        INPUTLANGCHANGE = 0x51,
        TCARD = 0x52,
        HELP = 0x53,
        USERCHANGED = 0x54,
        NOTIFYFORMAT = 0x55,
        CONTEXTMENU = 0x7B,
        STYLECHANGING = 0x7C,
        STYLECHANGED = 0x7D,
        DISPLAYCHANGE = 0x7E,
        GETICON = 0x7F,
        SETICON = 0x80,

        NCCREATE = 0x81,
        NCDESTROY = 0x82,
        NCCALCSIZE = 0x83,
        NCHITTEST = 0x84,
        NCPAINT = 0x85,
        NCACTIVATE = 0x86,
        GETDLGCODE = 0x87,
        NCMOUSEMOVE = 0xA0,
        NCLBUTTONDOWN = 0xA1,
        NCLBUTTONUP = 0xA2,
        NCLBUTTONDBLCLK = 0xA3,
        NCRBUTTONDOWN = 0xA4,
        NCRBUTTONUP = 0xA5,
        NCRBUTTONDBLCLK = 0xA6,
        NCMBUTTONDOWN = 0xA7,
        NCMBUTTONUP = 0xA8,
        NCMBUTTONDBLCLK = 0xA9,

        KEYFIRST = 0x100,
        KEYDOWN = 0x100,
        KEYUP = 0x101,
        CHAR = 0x102,
        DEADCHAR = 0x103,
        SYSKEYDOWN = 0x104,
        SYSKEYUP = 0x105,
        SYSCHAR = 0x106,
        SYSDEADCHAR = 0x107,
        KEYLAST = 0x108,

        IME_STARTCOMPOSITION = 0x10D,
        IME_ENDCOMPOSITION = 0x10E,
        IME_COMPOSITION = 0x10F,
        IME_KEYLAST = 0x10F,

        INITDIALOG = 0x110,
        COMMAND = 0x111,
        SYSCOMMAND = 0x112,
        TIMER = 0x113,
        HSCROLL = 0x114,
        VSCROLL = 0x115,
        INITMENU = 0x116,
        INITMENUPOPUP = 0x117,
        MENUSELECT = 0x11F,
        MENUCHAR = 0x120,
        ENTERIDLE = 0x121,

        CTLCOLORMSGBOX = 0x132,
        CTLCOLOREDIT = 0x133,
        CTLCOLORLISTBOX = 0x134,
        CTLCOLORBTN = 0x135,
        CTLCOLORDLG = 0x136,
        CTLCOLORSCROLLBAR = 0x137,
        CTLCOLORSTATIC = 0x138,

        MOUSEFIRST = 0x200,
        MOUSEMOVE = 0x200,
        LBUTTONDOWN = 0x201,
        LBUTTONUP = 0x202,
        LBUTTONDBLCLK = 0x203,
        RBUTTONDOWN = 0x204,
        RBUTTONUP = 0x205,
        RBUTTONDBLCLK = 0x206,
        MBUTTONDOWN = 0x207,
        MBUTTONUP = 0x208,
        MBUTTONDBLCLK = 0x209,
        MOUSEWHEEL = 0x20A,
        MOUSEHWHEEL = 0x20E,

        PARENTNOTIFY = 0x210,
        ENTERMENULOOP = 0x211,
        EXITMENULOOP = 0x212,
        NEXTMENU = 0x213,
        SIZING = 0x214,
        CAPTURECHANGED = 0x215,
        MOVING = 0x216,
        POWERBROADCAST = 0x218,
        DEVICECHANGE = 0x219,

        MDICREATE = 0x220,
        MDIDESTROY = 0x221,
        MDIACTIVATE = 0x222,
        MDIRESTORE = 0x223,
        MDINEXT = 0x224,
        MDIMAXIMIZE = 0x225,
        MDITILE = 0x226,
        MDICASCADE = 0x227,
        MDIICONARRANGE = 0x228,
        MDIGETACTIVE = 0x229,
        MDISETMENU = 0x230,
        ENTERSIZEMOVE = 0x231,
        EXITSIZEMOVE = 0x232,
        DROPFILES = 0x233,
        MDIREFRESHMENU = 0x234,

        IME_SETCONTEXT = 0x281,
        IME_NOTIFY = 0x282,
        IME_CONTROL = 0x283,
        IME_COMPOSITIONFULL = 0x284,
        IME_SELECT = 0x285,
        IME_CHAR = 0x286,
        IME_KEYDOWN = 0x290,
        IME_KEYUP = 0x291,

        MOUSEHOVER = 0x2A1,
        NCMOUSELEAVE = 0x2A2,
        MOUSELEAVE = 0x2A3,

        CUT = 0x300,
        COPY = 0x301,
        PASTE = 0x302,
        CLEAR = 0x303,
        UNDO = 0x304,

        RENDERFORMAT = 0x305,
        RENDERALLFORMATS = 0x306,
        DESTROYCLIPBOARD = 0x307,
        DRAWCLIPBOARD = 0x308,
        PAINTCLIPBOARD = 0x309,
        VSCROLLCLIPBOARD = 0x30A,
        SIZECLIPBOARD = 0x30B,
        ASKCBFORMATNAME = 0x30C,
        CHANGECBCHAIN = 0x30D,
        HSCROLLCLIPBOARD = 0x30E,
        QUERYNEWPALETTE = 0x30F,
        PALETTEISCHANGING = 0x310,
        PALETTECHANGED = 0x311,

        HOTKEY = 0x312,
        PRINT = 0x317,
        PRINTCLIENT = 0x318,

        HANDHELDFIRST = 0x358,
        HANDHELDLAST = 0x35F,
        PENWINFIRST = 0x380,
        PENWINLAST = 0x38F,
        COALESCE_FIRST = 0x390,
        COALESCE_LAST = 0x39F,
        DDE_FIRST = 0x3E0,
        DDE_INITIATE = 0x3E0,
        DDE_TERMINATE = 0x3E1,
        DDE_ADVISE = 0x3E2,
        DDE_UNADVISE = 0x3E3,
        DDE_ACK = 0x3E4,
        DDE_DATA = 0x3E5,
        DDE_REQUEST = 0x3E6,
        DDE_POKE = 0x3E7,
        DDE_EXECUTE = 0x3E8,
        DDE_LAST = 0x3E8,

        USER = 0x400,
        APP = 0x8000,
    } //enum WindowsMessage

} //namespace TestMessageDispatchingnamespace TestMessageDispatching {
